<?php

use Drupal\contact\Entity\ContactForm;
use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;

define('B_FIELDS_PER_BUNDLE', 10);
// The number of fields every single bundle shares.
define('B_SHARED_FIELDS_PER_BUNDLE', 2);
define('B_BUNDLES', 10);

if (B_SHARED_FIELDS_PER_BUNDLE > B_FIELDS_PER_BUNDLE) {
  exit('Shared fields per bundle cannot exceed fields per bundle.');
}

// Create field storage configs.
$unique_fields_per_bundle = B_FIELDS_PER_BUNDLE - B_SHARED_FIELDS_PER_BUNDLE;
$field_count = $unique_fields_per_bundle * B_BUNDLES + B_SHARED_FIELDS_PER_BUNDLE;
$storages = [];
$total_time = 0;
for ($i = 0; $i < $field_count; $i++) {
  $start_time = microtime(TRUE);
  $storages[$i] = FieldStorageConfig::create([
    'field_name' => 'field_map_storage_' . $i,
    'entity_type' => 'contact_message',
    'type' => 'text',
  ]);
  $storages[$i]->save();
  $end_time = microtime(TRUE);
  print "$i: Created field storage config in " . round($end_time - $start_time, 2) . "s.\n";
  $total_time += ($end_time - $start_time);
}

print "\nCreated $field_count field storage configs in " . round($total_time, 2) . "s.\n\n";

// Create bundles.
$total_time = 0;
for ($i = 0; $i < B_BUNDLES; $i++) {
  $start_time = microtime(TRUE);
  $form = ContactForm::create([
    'id' => 'field_map_' . $i,
    'label' => 'Contact form ' . $i,
  ])->save();
  $end_time = microtime(TRUE);
  print "$i: Created bundle in " . round($end_time - $start_time, 2) . "s.\n";
  $total_time += ($end_time - $start_time);
}

print "\nCreated " . B_BUNDLES . " bundles in " . round($total_time, 2) . "s.\n\n";

// Add field configs to bundles.
$total_time = 0;
$unique_counter = B_SHARED_FIELDS_PER_BUNDLE;
for ($i = 0; $i < B_BUNDLES; $i++) {
  // Add unique fields.
  $max = $unique_counter + $unique_fields_per_bundle;
  $start_time = microtime(TRUE);
  for ($unique_counter; $unique_counter < $max; $unique_counter++) {
    FieldConfig::create(array(
      'field_storage' => $storages[$unique_counter],
      'label' => 'Unique field #' . $unique_counter,
      'bundle' => 'field_map_' . $i,
    ))->save();
  }

  // Add shared fields.
  for ($k = 0; $k < B_SHARED_FIELDS_PER_BUNDLE; $k++) {
    FieldConfig::create(array(
      'field_storage' => $storages[$k],
      'label' => 'Shared field #' . $k,
      'bundle' => 'field_map_' . $i,
    ))->save();
  }

  $end_time = microtime(TRUE);
  print "$i: Added $unique_fields_per_bundle unique fields and " . B_SHARED_FIELDS_PER_BUNDLE . " shared fields to bundle in " . round($end_time - $start_time, 2) . "s.\n";
  $total_time += ($end_time - $start_time);
}

print "\nAdded $unique_fields_per_bundle unique fields and " .  B_SHARED_FIELDS_PER_BUNDLE . " shared fields to " . B_BUNDLES . " bundles in " . round($total_time, 2) . "s\n\n";

// Delete the bundles.
$total_time = 0;
for ($i = 0; $i < B_BUNDLES; $i++) {
  $start_time = microtime(TRUE);
  entity_delete_multiple('contact_form', array('field_map_' . $i));
  $end_time = microtime(TRUE);
  print "$i: Deleted bundle in " . round($end_time - $start_time, 2) . "s.\n";
  $total_time += ($end_time - $start_time);
}

print "\nDeleted " . B_BUNDLES . " bundles in " . round($total_time, 2) . "s\n\n";
